package com.huigou.topsun.customer.controller;

import com.huigou.topsun.customer.application.CustomerBaseInfoApplication;
import com.huigou.topsun.customer.application.CustomerInfoApplication;
import com.huigou.topsun.customer.domain.Province;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.client.CommonController;
import com.huigou.util.SDO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Controller
@ControllerMapping("/customerInfo")
public class CustomerInfoController extends CommonController {
    @Autowired
    private CustomerBaseInfoApplication customerBaseInfoApplication;
    @Autowired
    private CustomerInfoApplication customerInfoApplication;

    @Override
    protected String getPagePath() {
        return "/biz/topsun/customer/";
    }

    /**
     * 转跳到客户信息列表页面
     * @return
     */
    public String forwardCustomerInfoList() {
        return forward("customerInfoList");
    }

    /**
     * 展示客户信息列表
     * @return
     */
    public String findCustomerInfo() {
        List<Map<String,Object>> customerInfoPage = customerInfoApplication.findCustomerInfoPage(0,30);
        HashMap<Object, Object> map = new HashMap<>(3);
        map.put("Rows",customerInfoPage);
        return toResult(map);
    }

    /**
     * 转跳到客户列表页面
     * @return
     */
    public String forwardCustomerInfo() {
        return forward("customerInfoList");
    }

    /**
     * 转跳到客户详细信息页面
     * @return
     */
    public String forwardCustomerDetail() {
        return forward("customerInfoDetail");
    }
    /**
     * 展示客户省市数据，如果客户的省份编码是省级行政单位，直接返回省份数据，
     * 如果客户的省份编码是二级或者三级行政单位，则需要返回对应的省级或者市级行政单位
     * @return
     */
    public String findProvinceInfo() {
        SDO sdo = this.getSDO();
        //获取省份编码
        String provinceCode = sdo.getString("provinceCode");
        List<Province> provinceList = new ArrayList<>();
        if (!StringUtils.isEmpty(provinceCode)) {
            provinceList = customerBaseInfoApplication.findByProvinceCode(provinceCode);
        }
        List<String> provinceNameList = provinceList.stream().map(province -> province.getProvinceName()).collect(Collectors.toList());
        Map<Object, Object> map = new HashMap<>();
        map.put("provinceNameList",provinceNameList);
        return toResult(map);
    }

    /**
     * 客户详细信息展示
     * @return
     */
//    public String loadProductDetail() {
//
//    }


}
